"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CleanCommand = void 0;
const path = require("path");
const yok_1 = require("../common/yok");
const constants = require("../constants");
class CleanCommand {
    constructor($fs, $logger, $projectHelper, $projectConfigService, $terminalSpinnerService) {
        this.$fs = $fs;
        this.$logger = $logger;
        this.$projectHelper = $projectHelper;
        this.$projectConfigService = $projectConfigService;
        this.$terminalSpinnerService = $terminalSpinnerService;
        this.allowedParameters = [];
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const spinner = this.$terminalSpinnerService.createSpinner();
            spinner.start("Cleaning project...");
            const pathsToClean = [
                constants.HOOKS_DIR_NAME,
                constants.PLATFORMS_DIR_NAME,
                constants.NODE_MODULES_FOLDER_NAME,
                constants.PACKAGE_LOCK_JSON_FILE_NAME,
            ];
            try {
                const additionalPaths = this.$projectConfigService.getValue("additionalPathsToClean");
                if (Array.isArray(additionalPaths)) {
                    pathsToClean.push(...additionalPaths);
                }
                for (const pathToClean of pathsToClean) {
                    const filePath = path.resolve(this.$projectHelper.projectDir, pathToClean);
                    const displayPath = `${path.relative(this.$projectHelper.projectDir, filePath)}`.yellow;
                    spinner.start(`Cleaning ${displayPath}`);
                    this.$logger.trace(`Trying to clean '${filePath}'`);
                    if (this.$fs.exists(filePath)) {
                        const stat = this.$fs.getFsStats(filePath);
                        if (stat.isDirectory()) {
                            this.$logger.trace(`Path '${filePath}' is a directory, deleting.`);
                            this.$fs.deleteDirectorySafe(filePath);
                            spinner.text = `Cleaned directory ${displayPath}`;
                            spinner.succeed();
                        }
                        else {
                            this.$logger.trace(`Path '${filePath}' is a file, deleting.`);
                            this.$fs.deleteFile(filePath);
                            spinner.text = `Cleaned file ${displayPath}`;
                            spinner.succeed();
                        }
                    }
                    else {
                        spinner.text = `Skipping ${displayPath} because it doesn't exist.`;
                        spinner.info();
                        this.$logger.trace(`Path '${filePath}' not found, skipping.`);
                    }
                }
            }
            catch (err) {
                this.$logger.trace(`Encountered error while cleaning. Error is: ${err.message}.`, err);
            }
            spinner.succeed("Project successfully cleaned.");
        });
    }
}
exports.CleanCommand = CleanCommand;
yok_1.injector.registerCommand("clean", CleanCommand);
